"""This script creates the basic structure of a Basthon kernel project in the
current directory.
"""

from pathlib import Path
import shutil
import argparse
import sys

from . import implementation


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--install',
                        help='Install Basthon kernel in current directory',
                        action="store_true")

    parser.add_argument('--languages',
                        help='Get the list of kanguages supported in Basthon',
                        action="store_true")

    args = parser.parse_args()

    if args.languages:
        languages = [f.name[len('basthon-'):-len('.min.js')]
                     for f in (Path(__file__).parent / 'data').glob(
                             'basthon-*.min.js')]
        print('\n'.join(languages), end='')
        return

    if args.install:
        print(f'Installing Basthon kernel {implementation} in current directory')

        data_path = Path(__file__).parent / 'data'

        for path in data_path.rglob("*"):
            if path.is_file():
                shutil.copyfile(path, path.relative_to(data_path))
            elif path.is_dir():
                path.relative_to(data_path).mkdir(exist_ok=True)


if __name__ == "__main__":
    main()
