(function (root, factory) {
    // requirejs?
    if (typeof define === 'function') {
        define('Basthon', ['BasthonBase'], factory);
    // TypeScript?
    } else if (typeof exports === 'object') {
        module.exports = factory();
    // default goes to global namespace
    } else {
        root.Basthon = factory(BasthonBase);
    }
}(this, function (BasthonBase) {
    'use strict';

    class Basthon extends BasthonBase {
        constructor() {
            super();
            this.execution_count = 0;
            this.context = {Basthon: undefined,
                            Jupyter: undefined,
                            window: undefined,};
        };

        language () { return "javascript"; };
        languageName () { return "Javascript"; };
        moduleExts () { return ['js']; };

        async launch () { await this.pageLoad(); };

        eval (code) {
            try {
                // using context to hide several global variables
                return window.eval(`with(Basthon.context) { ${code} }`);
            } catch (e) {
                if (e instanceof SyntaxError)
                    return (new Function(`with(this) { ${code} }`)).call(this.context);
                throw e;
            }
        };
        
        async evalAsync (code, outCallback, errCallback, data=null) {
            // force interactivity in all modes
            data.interactive = true;

            // backup
            const console_log = console.log;
            const console_error = console.error;
            console.log = outCallback;
            console.error = errCallback;

            this.execution_count++;

            // evaluation
            let result;
            try {
                result = this.eval(code);
            } catch (e) {
                console.error(e.message);
            }

            // restoration
            console.log = console_log;
            console.error = console_error;

            // return result
            if( typeof result !== 'undefined' )
                result = {'text/plain': JSON.stringify(result)};
            return [result, this.execution_count];
        };

        ps1 () { return " js> "; };

        ps2 () { return "...> "; };

        restart () { };

        more (source) { return false; };
          
        complete (code) { return []; };

        putFile (filename, content) {
            console.error(`Fichier ${filename} not added since putFile has no mean in the JS context.`);
        };

        putModule (filename, content) {
            const ext = filename.split('/').pop();
            switch( ext ) {
            case 'js':
                this.eval(content);
                break;
            default:
                throw {message: "Only '.js' files supported."};
            }
        };
    }

    return new Basthon();
}));
